import asyncio
from concurrent.futures import ThreadPoolExecutor
from functools import partial

from hamilton.function_modifiers import extract_columns

MAX_THREADS = 10
THREAD_POOL = ThreadPoolExecutor(MAX_THREADS)

# async helper function for running queries and grabbing results
async def _async_run_query(query_string):
    # grabs the same event loop that Hamilton is using
    loop = asyncio.get_running_loop()

    # run_query() is synchronous/blocking so I execute it in a new thread
    partial_run_query = partial(run_query, query_string)
    results_s3_path = await loop.run_in_executor(THREAD_POOL, partial_run_query)
    
    # get_results() is also synchronous/blocking
    partial_get_results = partial(get_results, results_s3_path)
    df = await loop.run_in_executor(THREAD_POOL, partial_get_results)
    return df

# async Hamilton func for query #1
async def sql_1(...) -> pd.DataFrame:
    query_string = "..."
    return await _async_run_query(query_string)

# query #1 dataframe -> columns
@extract_columns(...)
def sql_1_extract(sql_1: pd.DataFrame) -> pd.DataFrame:
    return sql_1

# async Hamilton func for query #n
async def sql_n(...) -> pd.DataFrame:
    query_string = "..."
    return await _async_run_query(query_string)

# query #n dataframe -> columns
@extract_columns(...)
def sql_n_extract(sql_n: pd.DataFrame) -> pd.DataFrame:
    return sql_n
